@@ -5,7 +5,6 @@
 #![feature(drain_filter)]
 #![feature(nll)]
 #![feature(box_syntax)]
-#![feature(maybe_uninit_ref)]
 #![feature(maybe_uninit_extra)]
 #![feature(get_mut_unchecked)]
 #![feature(arbitrary_self_types)]
@@ -13,13 +12,22 @@
 #![feature(specialization)]
 #![feature(trait_alias)]
 #![feature(const_panic)]
-#![feature(const_fn_union)]
 #![feature(step_trait)]
-#![feature(step_trait_ext)]
-#![feature(const_generics)]
+#![feature(adt_const_params)]
+#![feature(generic_const_exprs)]
 // TODO: We should fix missing docs for public items and turn this on (Issue #309).
 // #![deny(missing_docs)]
 
+// Allow this for now. Clippy suggests we should use Sft, Mmtk, rather than SFT and MMTK.
+// According to its documentation (https://rust-lang.github.io/rust-clippy/master/index.html#upper_case_acronyms),
+// with upper-case-acronyms-aggressive turned on, it should also warn us about SFTMap, VMBinding, GCWorker.
+// However, it seems clippy does not catch all these patterns at the moment. So it would be hard for us to
+// find all the patterns and consistently change all of them. I think it would be a better idea to just allow this.
+// We may reconsider this in the future. Plus, using upper case letters for acronyms does not sound a big issue
+// to me - considering it will break our API and all the efforts for all the developers to make the change, it may
+// not worth it.
+#![allow(clippy::upper_case_acronyms)]
+
 //! Memory Management ToolKit (MMTk) is a portable and high performance memory manager
 //! that includes various garbage collection algorithms and provides clean and efficient
 //! interfaces to cooperate with language implementations. MMTk features highly modular
