@@ -14,9 +14,8 @@
 use crate::mmtk::MMTK;
 use crate::plan::AllocationSemantics;
 use crate::plan::{Mutator, MutatorContext};
-use crate::scheduler::GCWorker;
-use crate::scheduler::Work;
 use crate::scheduler::WorkBucketStage;
+use crate::scheduler::{GCWork, GCWorker};
 use crate::util::alloc::allocators::AllocatorSelector;
 use crate::util::constants::{LOG_BYTES_IN_PAGE, MIN_OBJECT_SIZE};
 use crate::util::heap::layout::vm_layout_constants::HEAP_END;
@@ -411,7 +410,7 @@ pub fn num_of_workers<VM: VMBinding>(mmtk: &'static MMTK<VM>) -> usize {
 /// * `mmtk`: A reference to an MMTk instance.
 /// * `bucket`: Which work bucket to add this packet to.
 /// * `packet`: The work packet to be added.
-pub fn add_work_packet<VM: VMBinding, W: Work<MMTK<VM>>>(
+pub fn add_work_packet<VM: VMBinding, W: GCWork<VM>>(
     mmtk: &'static MMTK<VM>,
     bucket: WorkBucketStage,
     packet: W,
@@ -429,7 +428,7 @@ pub fn add_work_packet<VM: VMBinding, W: Work<MMTK<VM>>>(
 pub fn add_work_packets<VM: VMBinding>(
     mmtk: &'static MMTK<VM>,
     bucket: WorkBucketStage,
-    packets: Vec<Box<dyn Work<MMTK<VM>>>>,
+    packets: Vec<Box<dyn GCWork<VM>>>,
 ) {
     mmtk.scheduler.work_buckets[bucket].bulk_add(packets)
 }
