@@ -3,6 +3,7 @@
 use super::controller_collector_context::ControllerCollectorContext;
 use super::PlanConstraints;
 use crate::mmtk::MMTK;
+use crate::plan::generational::global::Gen;
 use crate::plan::transitive_closure::TransitiveClosure;
 use crate::plan::Mutator;
 use crate::policy::immortalspace::ImmortalSpace;
@@ -107,7 +108,7 @@ impl<VM: VMBinding> NoCopy<VM> {
     }
 }
 
-impl<VM: VMBinding> WorkerLocal for NoCopy<VM> {
+impl<VM: VMBinding> GCWorkerLocal for NoCopy<VM> {
     fn init(&mut self, tls: VMWorkerThread) {
         CopyContext::init(self, tls);
     }
@@ -122,7 +123,12 @@ pub fn create_mutator<VM: VMBinding>(
         PlanSelector::SemiSpace => {
             crate::plan::semispace::mutator::create_ss_mutator(tls, &*mmtk.plan)
         }
-        PlanSelector::GenCopy => crate::plan::gencopy::mutator::create_gencopy_mutator(tls, mmtk),
+        PlanSelector::GenCopy => {
+            crate::plan::generational::copying::mutator::create_gencopy_mutator(tls, mmtk)
+        }
+        PlanSelector::GenImmix => {
+            crate::plan::generational::immix::mutator::create_genimmix_mutator(tls, mmtk)
+        }
         PlanSelector::MarkSweep => {
             crate::plan::marksweep::mutator::create_ms_mutator(tls, &*mmtk.plan)
         }
@@ -138,16 +144,19 @@ pub fn create_plan<VM: VMBinding>(
     vm_map: &'static VMMap,
     mmapper: &'static Mmapper,
     options: Arc<UnsafeOptionsWrapper>,
-    scheduler: Arc<MMTkScheduler<VM>>,
+    scheduler: Arc<GCWorkScheduler<VM>>,
 ) -> Box<dyn Plan<VM = VM>> {
     match plan {
         PlanSelector::NoGC => Box::new(crate::plan::nogc::NoGC::new(vm_map, mmapper, options)),
         PlanSelector::SemiSpace => Box::new(crate::plan::semispace::SemiSpace::new(
             vm_map, mmapper, options,
         )),
-        PlanSelector::GenCopy => {
-            Box::new(crate::plan::gencopy::GenCopy::new(vm_map, mmapper, options))
-        }
+        PlanSelector::GenCopy => Box::new(crate::plan::generational::copying::GenCopy::new(
+            vm_map, mmapper, options,
+        )),
+        PlanSelector::GenImmix => Box::new(crate::plan::generational::immix::GenImmix::new(
+            vm_map, mmapper, options, scheduler,
+        )),
         PlanSelector::MarkSweep => Box::new(crate::plan::marksweep::MarkSweep::new(
             vm_map, mmapper, options,
         )),
@@ -186,10 +195,13 @@ pub trait Plan: 'static + Sync + Downcast {
         mmtk: &'static MMTK<Self::VM>,
     ) -> GCWorkerLocalPtr;
     fn base(&self) -> &BasePlan<Self::VM>;
-    fn schedule_collection(&'static self, _scheduler: &MMTkScheduler<Self::VM>);
+    fn schedule_collection(&'static self, _scheduler: &GCWorkScheduler<Self::VM>);
     fn common(&self) -> &CommonPlan<Self::VM> {
         panic!("Common Plan not handled!")
     }
+    fn generational(&self) -> &Gen<Self::VM> {
+        panic!("This is not a generational plan.")
+    }
     fn mmapper(&self) -> &'static Mmapper {
         self.base().mmapper
     }
@@ -202,7 +214,7 @@ pub trait Plan: 'static + Sync + Downcast {
         &mut self,
         heap_size: usize,
         vm_map: &'static VMMap,
-        scheduler: &Arc<MMTkScheduler<Self::VM>>,
+        scheduler: &Arc<GCWorkScheduler<Self::VM>>,
     );
 
     fn get_allocator_mapping(&self) -> &'static EnumMap<AllocationSemantics, AllocatorSelector>;
@@ -506,7 +518,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         &mut self,
         heap_size: usize,
         vm_map: &'static VMMap,
-        scheduler: &Arc<MMTkScheduler<VM>>,
+        scheduler: &Arc<GCWorkScheduler<VM>>,
     ) {
         vm_map.boot();
         vm_map.finalize_static_space_map(
@@ -583,7 +595,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         panic!("No special case for space in trace_object({:?})", _object);
     }
 
-    pub fn prepare(&mut self, _tls: VMWorkerThread, _primary: bool) {
+    pub fn prepare(&mut self, _tls: VMWorkerThread, _full_heap: bool) {
         #[cfg(feature = "code_space")]
         self.code_space.prepare();
         #[cfg(feature = "code_space")]
@@ -594,7 +606,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         self.vm_space.prepare();
     }
 
-    pub fn release(&mut self, _tls: VMWorkerThread, _primary: bool) {
+    pub fn release(&mut self, _tls: VMWorkerThread, _full_heap: bool) {
         #[cfg(feature = "code_space")]
         self.code_space.release();
         #[cfg(feature = "code_space")]
@@ -798,7 +810,7 @@ impl<VM: VMBinding> CommonPlan<VM> {
         &mut self,
         heap_size: usize,
         vm_map: &'static VMMap,
-        scheduler: &Arc<MMTkScheduler<VM>>,
+        scheduler: &Arc<GCWorkScheduler<VM>>,
     ) {
         self.base.gc_init(heap_size, vm_map, scheduler);
         self.immortal.init(vm_map);
@@ -825,22 +837,22 @@ impl<VM: VMBinding> CommonPlan<VM> {
         self.base.trace_object::<T, C>(trace, object)
     }
 
-    pub fn prepare(&mut self, tls: VMWorkerThread, primary: bool) {
+    pub fn prepare(&mut self, tls: VMWorkerThread, full_heap: bool) {
         self.immortal.prepare();
-        self.los.prepare(primary);
-        self.base.prepare(tls, primary)
+        self.los.prepare(full_heap);
+        self.base.prepare(tls, full_heap)
     }
 
-    pub fn release(&mut self, tls: VMWorkerThread, primary: bool) {
+    pub fn release(&mut self, tls: VMWorkerThread, full_heap: bool) {
         self.immortal.release();
-        self.los.release(primary);
-        self.base.release(tls, primary)
+        self.los.release(full_heap);
+        self.base.release(tls, full_heap)
     }
 
     pub fn schedule_common<E: ProcessEdgesWork<VM = VM>>(
         &self,
         constraints: &'static PlanConstraints,
-        scheduler: &MMTkScheduler<VM>,
+        scheduler: &GCWorkScheduler<VM>,
     ) {
         // Schedule finalization
         if !self.base.options.no_finalizer {
