@@ -54,7 +54,12 @@ impl<VM: VMBinding> Allocator<VM> for ImmixAllocator<VM> {
 
     #[inline(always)]
     fn alloc(&mut self, size: usize, align: usize, offset: isize) -> Address {
-        trace!("alloc");
+        debug_assert!(
+            size <= crate::policy::immix::MAX_IMMIX_OBJECT_SIZE,
+            "Trying to allocate a {} bytes object, which is larger than MAX_IMMIX_OBJECT_SIZE {}",
+            size,
+            crate::policy::immix::MAX_IMMIX_OBJECT_SIZE
+        );
         let result = align_allocation_no_fill::<VM>(self.cursor, align, offset);
         let new_cursor = result + size;
 
@@ -172,6 +177,8 @@ impl<VM: VMBinding> ImmixAllocator<VM> {
                     lines,
                     self.tls
                 );
+                #[cfg(feature = "global_alloc_bit")]
+                crate::util::alloc_bit::bzero_alloc_bit(self.cursor, self.limit - self.cursor);
                 crate::util::memory::zero(self.cursor, self.limit - self.cursor);
                 debug_assert!(
                     align_allocation_no_fill::<VM>(self.cursor, align, offset) + size <= self.limit
