@@ -13,16 +13,20 @@ use super::metadata_address_range_size;
 // XXX: We updated the base address to start from the second 4Mb chunk for 32-bit architectures,
 // as otherwise for side metadatas with a large `min_obj_size`, we were overlapping with system
 // reserved addresses such as 0x0.
+// public, VM bingdings may need to use this
 #[cfg(target_pointer_width = "32")]
-pub(crate) const GLOBAL_SIDE_METADATA_BASE_ADDRESS: Address =
+pub const GLOBAL_SIDE_METADATA_BASE_ADDRESS: Address =
     unsafe { Address::from_usize(BYTES_IN_CHUNK) };
 #[cfg(target_pointer_width = "64")]
-pub(crate) const GLOBAL_SIDE_METADATA_BASE_ADDRESS: Address =
+pub const GLOBAL_SIDE_METADATA_BASE_ADDRESS: Address =
     unsafe { Address::from_usize(0x0000_0600_0000_0000usize) };
 
 pub(crate) const GLOBAL_SIDE_METADATA_BASE_OFFSET: SideMetadataOffset =
     SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS);
 
+// Base address of alloc bit, public to VM bindings which may need to use this.
+pub const ALLOC_SIDE_METADATA_ADDR: Address = crate::util::alloc_bit::ALLOC_SIDE_METADATA_ADDR;
+
 /// This constant represents the worst-case ratio of source data size to global side metadata.
 /// A value of 2 means the space required for global side metadata must be less than 1/4th of the source data.
 /// So, a value of `n` means this ratio must be less than $2^-n$.
@@ -39,7 +43,7 @@ pub(super) const LOG_LOCAL_SIDE_METADATA_WORST_CASE_RATIO: usize = 3;
 #[cfg(target_pointer_width = "64")]
 pub(super) const LOG_LOCAL_SIDE_METADATA_WORST_CASE_RATIO: usize = 1;
 
-const LOG_MAX_GLOBAL_SIDE_METADATA_SIZE: usize =
+pub const LOG_MAX_GLOBAL_SIDE_METADATA_SIZE: usize =
     LOG_ADDRESS_SPACE - LOG_GLOBAL_SIDE_METADATA_WORST_CASE_RATIO;
 // TODO - we should check this limit somewhere
 // pub(crate) const LOG_MAX_LOCAL_SIDE_METADATA_SIZE: usize =
@@ -77,15 +81,22 @@ pub(super) const LOCAL_SIDE_METADATA_PER_CHUNK: usize =
 //
 // MMTk reserved Global side metadata offsets:
 //
-//  1 - MarkSweep Active Chunk byte: 1 byte per chunk, used by malloc marksweep.
+//  1 - Global Allocation bit:  1 bit per object, used by MarkSweep OR when enabling the global_alloc_bit feature
+//  2 - MarkSweep Active Chunk byte: 1 byte per chunk, used by malloc marksweep.
 //
 // --------------------------------------------------
 
 /// The base address for the global side metadata space available to VM bindings, to be used for the per-object metadata.
 /// VM bindings must use this to avoid overlap with core internal global side metadata.
-pub const GLOBAL_SIDE_METADATA_VM_BASE_ADDRESS: Address = GLOBAL_SIDE_METADATA_BASE_ADDRESS.add(
-    metadata_address_range_size(&crate::policy::mallocspace::metadata::ACTIVE_CHUNK_METADATA_SPEC),
-);
+
+pub const GLOBAL_SIDE_METADATA_VM_BASE_ADDRESS: Address = GLOBAL_SIDE_METADATA_BASE_ADDRESS
+    .add(metadata_address_range_size(
+        &crate::util::alloc_bit::ALLOC_SIDE_METADATA_SPEC,
+    ))
+    .add(metadata_address_range_size(
+        &crate::policy::mallocspace::metadata::ACTIVE_CHUNK_METADATA_SPEC,
+    ));
+
 pub const GLOBAL_SIDE_METADATA_VM_BASE_OFFSET: SideMetadataOffset =
     SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_VM_BASE_ADDRESS);
 
@@ -94,17 +105,15 @@ pub const GLOBAL_SIDE_METADATA_VM_BASE_OFFSET: SideMetadataOffset =
 //
 // MMTk reserved PolicySpecific side metadata offsets:
 //
-//  1 - MarkSweep Alloc bit:
-//      - Offset LOCAL_SIDE_METADATA_BASE_OFFSET
-//  2 - MarkSweep Active Page byte:
-//      - Offset after Alloc bit
-//  3 - Immix line mark byte:
-//      - Offset after MS alloc bit
-//  4 - Immix block defrag byte:
+//  1 - MarkSweep Active Page byte:
+//      -
+//  2 - Immix line mark byte:
+//      - Offset after MS Active Page byte
+//  3 - Immix block defrag byte:
 //      - Offset after Immix block defrag byte
-//  5 - Immix block mark byte:
+//  4 - Immix block mark byte:
 //      - Offset after Immix block mark byte
-//  6 - Immix chumk-map mark byte:
+//  5 - Immix chumk-map mark byte:
 //      - Offset after Immix chumk-map mark byte
 //
 // --------------------------------------------------
