@@ -14,14 +14,14 @@ mod tests {
     fn test_side_metadata_address_to_meta_address() {
         let mut gspec = SideMetadataSpec {
             is_global: true,
-            offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         #[cfg(target_pointer_width = "64")]
         let mut lspec = SideMetadataSpec {
             is_global: false,
-            offset: LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -29,96 +29,96 @@ mod tests {
         #[cfg(target_pointer_width = "32")]
         let mut lspec = SideMetadataSpec {
             is_global: false,
-            offset: 0,
+            offset: SideMetadataOffset::rel(0),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(0) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(0) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(0) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(0) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(7) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(7) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(7) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(7) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(27) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 3
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(27) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS + 3usize
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(129) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 16
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(129) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS + 16usize
         );
 
         gspec.log_min_obj_size = 2;
         lspec.log_min_obj_size = 1;
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(0) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(0) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(0) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(0) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(32) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 1
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(32) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS + 1usize
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(32) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 2
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(32) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS + 2usize
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(316) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 9
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(316) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS + 9usize
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(316) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 19
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(316) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS + 19usize
         );
 
         gspec.log_num_of_bits = 1;
         lspec.log_num_of_bits = 3;
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(0) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(0) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(0) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize()
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(0) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(32) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 2
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(32) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS + 2usize
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(32) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 16
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(32) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS + 16usize
         );
 
         assert_eq!(
-            address_to_meta_address(gspec, unsafe { Address::from_usize(316) }).as_usize(),
-            GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 19
+            address_to_meta_address(&gspec, unsafe { Address::from_usize(316) }),
+            GLOBAL_SIDE_METADATA_BASE_ADDRESS + 19usize
         );
         assert_eq!(
-            address_to_meta_address(lspec, unsafe { Address::from_usize(318) }).as_usize(),
-            LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize() + 159
+            address_to_meta_address(&lspec, unsafe { Address::from_usize(318) }),
+            LOCAL_SIDE_METADATA_BASE_ADDRESS + 159usize
         );
     }
 
@@ -126,47 +126,59 @@ mod tests {
     fn test_side_metadata_meta_byte_mask() {
         let mut spec = SideMetadataSpec {
             is_global: true,
-            offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
 
-        assert_eq!(meta_byte_mask(spec), 1);
+        assert_eq!(meta_byte_mask(&spec), 1);
 
         spec.log_num_of_bits = 1;
-        assert_eq!(meta_byte_mask(spec), 3);
+        assert_eq!(meta_byte_mask(&spec), 3);
         spec.log_num_of_bits = 2;
-        assert_eq!(meta_byte_mask(spec), 15);
+        assert_eq!(meta_byte_mask(&spec), 15);
         spec.log_num_of_bits = 3;
-        assert_eq!(meta_byte_mask(spec), 255);
+        assert_eq!(meta_byte_mask(&spec), 255);
     }
 
     #[test]
     fn test_side_metadata_meta_byte_lshift() {
         let mut spec = SideMetadataSpec {
             is_global: true,
-            offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
 
-        assert_eq!(meta_byte_lshift(spec, unsafe { Address::from_usize(0) }), 0);
-        assert_eq!(meta_byte_lshift(spec, unsafe { Address::from_usize(5) }), 5);
         assert_eq!(
-            meta_byte_lshift(spec, unsafe { Address::from_usize(15) }),
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(0) }),
+            0
+        );
+        assert_eq!(
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(5) }),
+            5
+        );
+        assert_eq!(
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(15) }),
             7
         );
 
         spec.log_num_of_bits = 2;
 
-        assert_eq!(meta_byte_lshift(spec, unsafe { Address::from_usize(0) }), 0);
-        assert_eq!(meta_byte_lshift(spec, unsafe { Address::from_usize(5) }), 4);
         assert_eq!(
-            meta_byte_lshift(spec, unsafe { Address::from_usize(15) }),
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(0) }),
+            0
+        );
+        assert_eq!(
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(5) }),
             4
         );
         assert_eq!(
-            meta_byte_lshift(spec, unsafe { Address::from_usize(0x10010) }),
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(15) }),
+            4
+        );
+        assert_eq!(
+            meta_byte_lshift(&spec, unsafe { Address::from_usize(0x10010) }),
             0
         );
     }
@@ -180,21 +192,21 @@ mod tests {
                     // sanity::reset();
                     let mut gspec = SideMetadataSpec {
                         is_global: true,
-                        offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+                        offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
                         log_min_obj_size: 1,
                     };
                     #[cfg(target_pointer_width = "64")]
                     let mut lspec = SideMetadataSpec {
                         is_global: false,
-                        offset: LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+                        offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
                         log_min_obj_size: 1,
                     };
                     #[cfg(target_pointer_width = "32")]
                     let mut lspec = SideMetadataSpec {
                         is_global: false,
-                        offset: 0,
+                        offset: SideMetadataOffset::rel(0),
                         log_num_of_bits: 1,
                         log_min_obj_size: 1,
                     };
@@ -214,14 +226,14 @@ mod tests {
                         )
                         .is_ok());
 
-                    ensure_metadata_is_mapped(gspec, vm_layout_constants::HEAP_START);
-                    ensure_metadata_is_mapped(lspec, vm_layout_constants::HEAP_START);
+                    ensure_metadata_is_mapped(&gspec, vm_layout_constants::HEAP_START);
+                    ensure_metadata_is_mapped(&lspec, vm_layout_constants::HEAP_START);
                     ensure_metadata_is_mapped(
-                        gspec,
+                        &gspec,
                         vm_layout_constants::HEAP_START + constants::BYTES_IN_PAGE - 1,
                     );
                     ensure_metadata_is_mapped(
-                        lspec,
+                        &lspec,
                         vm_layout_constants::HEAP_START + constants::BYTES_IN_PAGE - 1,
                     );
 
@@ -253,20 +265,20 @@ mod tests {
                         .is_ok());
 
                     ensure_metadata_is_mapped(
-                        gspec,
+                        &gspec,
                         vm_layout_constants::HEAP_START + vm_layout_constants::BYTES_IN_CHUNK,
                     );
                     ensure_metadata_is_mapped(
-                        lspec,
+                        &lspec,
                         vm_layout_constants::HEAP_START + vm_layout_constants::BYTES_IN_CHUNK,
                     );
                     ensure_metadata_is_mapped(
-                        gspec,
+                        &gspec,
                         vm_layout_constants::HEAP_START + vm_layout_constants::BYTES_IN_CHUNK * 2
                             - 8,
                     );
                     ensure_metadata_is_mapped(
-                        lspec,
+                        &lspec,
                         vm_layout_constants::HEAP_START + vm_layout_constants::BYTES_IN_CHUNK * 2
                             - 16,
                     );
@@ -294,15 +306,14 @@ mod tests {
 
                     let metadata_1_spec = SideMetadataSpec {
                         is_global: true,
-                        offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+                        offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 4,
                         log_min_obj_size: 6,
                     };
 
                     let metadata_2_spec = SideMetadataSpec {
                         is_global: true,
-                        offset: metadata_1_spec.offset
-                            + metadata_address_range_size(&metadata_1_spec),
+                        offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
                         log_min_obj_size: 7,
                     };
@@ -319,30 +330,30 @@ mod tests {
                         .try_map_metadata_space(data_addr, constants::BYTES_IN_PAGE,)
                         .is_ok());
 
-                    let zero = fetch_add_atomic(metadata_1_spec, data_addr, 5, Ordering::SeqCst);
+                    let zero = fetch_add_atomic(&metadata_1_spec, data_addr, 5, Ordering::SeqCst);
                     assert_eq!(zero, 0);
 
-                    let five = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 5);
 
-                    let zero = fetch_add_atomic(metadata_2_spec, data_addr, 5, Ordering::SeqCst);
+                    let zero = fetch_add_atomic(&metadata_2_spec, data_addr, 5, Ordering::SeqCst);
                     assert_eq!(zero, 0);
 
-                    let five = load_atomic(metadata_2_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_2_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 5);
 
                     let another_five =
-                        fetch_sub_atomic(metadata_1_spec, data_addr, 2, Ordering::SeqCst);
+                        fetch_sub_atomic(&metadata_1_spec, data_addr, 2, Ordering::SeqCst);
                     assert_eq!(another_five, 5);
 
-                    let three = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let three = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(three, 3);
 
                     let another_five =
-                        fetch_sub_atomic(metadata_2_spec, data_addr, 2, Ordering::SeqCst);
+                        fetch_sub_atomic(&metadata_2_spec, data_addr, 2, Ordering::SeqCst);
                     assert_eq!(another_five, 5);
 
-                    let three = load_atomic(metadata_2_spec, data_addr, Ordering::SeqCst);
+                    let three = load_atomic(&metadata_2_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(three, 3);
 
                     metadata.ensure_unmap_metadata_space(data_addr, constants::BYTES_IN_PAGE);
@@ -367,7 +378,7 @@ mod tests {
 
                     let metadata_1_spec = SideMetadataSpec {
                         is_global: true,
-                        offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+                        offset: SideMetadataOffset::addr(GLOBAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 1,
                         log_min_obj_size: constants::LOG_BYTES_IN_WORD as usize,
                     };
@@ -384,17 +395,17 @@ mod tests {
                         .try_map_metadata_space(data_addr, constants::BYTES_IN_PAGE,)
                         .is_ok());
 
-                    let zero = fetch_add_atomic(metadata_1_spec, data_addr, 2, Ordering::SeqCst);
+                    let zero = fetch_add_atomic(&metadata_1_spec, data_addr, 2, Ordering::SeqCst);
                     assert_eq!(zero, 0);
 
-                    let two = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let two = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(two, 2);
 
                     let another_two =
-                        fetch_sub_atomic(metadata_1_spec, data_addr, 1, Ordering::SeqCst);
+                        fetch_sub_atomic(&metadata_1_spec, data_addr, 1, Ordering::SeqCst);
                     assert_eq!(another_two, 2);
 
-                    let one = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let one = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(one, 1);
 
                     metadata.ensure_unmap_metadata_space(data_addr, constants::BYTES_IN_PAGE);
@@ -421,15 +432,14 @@ mod tests {
                     #[cfg(target_pointer_width = "64")]
                     let metadata_1_spec = SideMetadataSpec {
                         is_global: false,
-                        offset: LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+                        offset: SideMetadataOffset::addr(LOCAL_SIDE_METADATA_BASE_ADDRESS),
                         log_num_of_bits: 4,
                         log_min_obj_size: 9,
                     };
                     #[cfg(target_pointer_width = "64")]
                     let metadata_2_spec = SideMetadataSpec {
                         is_global: false,
-                        offset: metadata_1_spec.offset
-                            + metadata_address_range_size(&metadata_1_spec) as usize,
+                        offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
                         log_min_obj_size: 7,
                     };
@@ -437,18 +447,14 @@ mod tests {
                     #[cfg(target_pointer_width = "32")]
                     let metadata_1_spec = SideMetadataSpec {
                         is_global: false,
-                        offset: 0,
+                        offset: SideMetadataOffset::rel(0),
                         log_num_of_bits: 4,
                         log_min_obj_size: 9,
                     };
                     #[cfg(target_pointer_width = "32")]
                     let metadata_2_spec = SideMetadataSpec {
                         is_global: false,
-                        offset: metadata_1_spec.offset
-                            + metadata_bytes_per_chunk(
-                                metadata_1_spec.log_min_obj_size,
-                                metadata_1_spec.log_num_of_bits,
-                            ),
+                        offset: SideMetadataOffset::layout_after(&metadata_1_spec),
                         log_num_of_bits: 3,
                         log_min_obj_size: 7,
                     };
@@ -465,30 +471,30 @@ mod tests {
                         .try_map_metadata_space(data_addr, constants::BYTES_IN_PAGE,)
                         .is_ok());
 
-                    let zero = fetch_add_atomic(metadata_1_spec, data_addr, 5, Ordering::SeqCst);
+                    let zero = fetch_add_atomic(&metadata_1_spec, data_addr, 5, Ordering::SeqCst);
                     assert_eq!(zero, 0);
 
-                    let five = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 5);
 
-                    let zero = fetch_add_atomic(metadata_2_spec, data_addr, 5, Ordering::SeqCst);
+                    let zero = fetch_add_atomic(&metadata_2_spec, data_addr, 5, Ordering::SeqCst);
                     assert_eq!(zero, 0);
 
-                    let five = load_atomic(metadata_2_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_2_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 5);
 
-                    bzero_metadata(metadata_2_spec, data_addr, constants::BYTES_IN_PAGE);
+                    bzero_metadata(&metadata_2_spec, data_addr, constants::BYTES_IN_PAGE);
 
-                    let five = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 5);
-                    let five = load_atomic(metadata_2_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_2_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 0);
 
-                    bzero_metadata(metadata_1_spec, data_addr, constants::BYTES_IN_PAGE);
+                    bzero_metadata(&metadata_1_spec, data_addr, constants::BYTES_IN_PAGE);
 
-                    let five = load_atomic(metadata_1_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_1_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 0);
-                    let five = load_atomic(metadata_2_spec, data_addr, Ordering::SeqCst);
+                    let five = load_atomic(&metadata_2_spec, data_addr, Ordering::SeqCst);
                     assert_eq!(five, 0);
 
                     metadata.ensure_unmap_metadata_space(data_addr, constants::BYTES_IN_PAGE);
