@@ -2,10 +2,14 @@ use std::time::Instant;
 
 mod event_counter;
 mod long_counter;
+#[cfg(feature = "perf_counter")]
+mod perf_event;
 mod size_counter;
 
 pub use self::event_counter::EventCounter;
 pub use self::long_counter::{LongCounter, Timer};
+#[cfg(feature = "perf_counter")]
+pub use self::perf_event::PerfEventDiffable;
 pub use self::size_counter::SizeCounter;
 
 pub trait Counter {
@@ -22,9 +26,11 @@ pub trait Counter {
     fn name(&self) -> &String;
 }
 
+/// A Diffable object could be stateless (e.g. a timer that reads the wall
+/// clock), or stateful (e.g. holds reference to a perf event fd)
 pub trait Diffable {
     type Val;
-    fn current_value() -> Self::Val;
+    fn current_value(&mut self) -> Self::Val;
     fn diff(current: &Self::Val, earlier: &Self::Val) -> u64;
     fn print_diff(val: u64);
 }
@@ -34,7 +40,7 @@ pub struct MonotoneNanoTime;
 impl Diffable for MonotoneNanoTime {
     type Val = Instant;
 
-    fn current_value() -> Instant {
+    fn current_value(&mut self) -> Instant {
         Instant::now()
     }
 
